//
//  GSInstance.h
//  Glyphs
//
//  Created by Georg Seifert on 5.2.08.
//  Copyright 2008 schriftgestaltung.de. All rights reserved.
//

#import <Cocoa/Cocoa.h>

@class GSFont;
/** The class defining the instance object
 */

@interface GSInstance : NSObject <NSCoding, NSCopying> {
	GSFont		*__unsafe_unretained _font;
	NSString	*_name;
	NSString	*_linkStyle;
	float		_interpolationWeight;
	float		_interpolationWidth;
	float		_interpolationCustom;
	NSString	*_weightClass;
	NSString	*_widthClass;
	BOOL		_active;
	BOOL		_isItalic;
	BOOL		_isBold;
	NSMutableDictionary * _instanceInterpolations;
	NSMutableDictionary * tempData;
	NSMutableArray *_customParameters;
}
@property(unsafe_unretained, nonatomic) GSFont *font;
/** The Intances name
 
 This is used as the style name.
 */
@property(copy, nonatomic) NSString* name;
/** The style to use as a the regular for this style.
 
 if styleName is `Medium`, you could link to `Light`. So this instance becomes the Bold stil for the Light.
 */
@property(copy, nonatomic) NSString* linkStyle;
/// The interpolation Weight position.
@property(nonatomic) float interpolationWeight;
/// The interpolation Width position
@property(nonatomic) float interpolationWidth;
/// The interpolation Cutom position
@property(nonatomic) float interpolationCustom;
/** Weight Class
 
 e.g. Regular or Bold
 */
@property (strong, nonatomic) NSString* weightClass;
@property (nonatomic) NSUInteger weightClassValue;

/** Width Class
 
 e.g. Medium (normal)
 */
@property (strong, nonatomic) NSString* widthClass;
@property (nonatomic) NSUInteger widthClassValue;

/// If it should be exported.
@property (nonatomic) BOOL active;
/// the Style linking Italic bit
@property (nonatomic) BOOL isItalic;
/// the Style linking Bold bit
@property (nonatomic) BOOL isBold;
@property(strong, nonatomic) NSMutableDictionary * instanceInterpolations;
/// The content of the instance to store it in a pList.
@property(unsafe_unretained) NSDictionary *instanceDict;
@property(strong, nonatomic) NSMutableDictionary *tempData;
@property(strong, nonatomic) NSMutableArray *customParameters;

- (id) initWithInstance:(GSInstance*) Instance;
#ifndef GLYPHS_VIEWER
- (BOOL) saveToFile:(FILE*) File error:(NSError**) error ;
#endif
- (NSArray *)customParameters;
- (NSUInteger)countOfCustomParameters;
- (id) objectInCustomParametersAtIndex:(NSUInteger)theIndex;
- (id) customParameterForKey:(NSString*) Key ;
//- (void) getCustomParameters:(id *)objsPtr range:(NSRange)range;
- (void) insertObject:(id)obj inCustomParametersAtIndex:(NSUInteger)theIndex;
- (void) removeObjectFromCustomParametersAtIndex:(NSUInteger)theIndex;
//- (void) addValueToCustomParameters:(id) Value forKey:(NSString*) Key;
- (void) removeObjectFromCustomParametersForKey:(NSString*) Key;

- (void) updateInterpolationValues ;
//- (void)replaceObjectInCustomParametersAtIndex:(NSUInteger)theIndex withObject:(id)obj;


@end
